@@ -81,7 +81,10 @@ class XRecordFK:
         return self.value.__cmp__(obj)
     def __repr__(self):
         return str(self.value)
-
+    def __add__(self, other):
+        return self.value.__add__(other)
+    def __radd__(self, other):
+        return other.__add__(self.value)
     
     def set_value(self, value):
         print "set_value", self.column, value
@@ -174,6 +177,7 @@ class XRecordMTMList(UserList.UserList):
             fk = value
         via_record = self._record.DB.XRecord ( mtm["via"], ** { str(mtm['via_to_column']) : fk, str(mtm['via_my_column']) : self._record[mtm['my_column']] } )
         via_record.insert()
+        self.nullify()
         return via_record
 
     def delete(self, other_record):
@@ -292,7 +296,7 @@ class XRecordDatabase:
 
         self.value_converters = {}
         self.server_side_converter = {}        
-        
+
         class XRecord:
             """
             Base class for all XRecords (active records).
@@ -353,14 +357,19 @@ class XRecordDatabase:
                     if key in self.META.conflicting_columns:
                         return self.META.conflicting_columns[key]
                     return getattr(self, key)
+                elif key in self.META.conflicting_columns:
+                    return self.META.conflicting_columns[key]
                 elif key in self.META.extra_data:
-                    return self.META.extra_data[key]
+                    return self.META.extra_data[key]                
                 else:
-                    return None
+                    try:
+                        return getattr(self, key)
+                    except AttributeError:
+                        raise IndexError(key)
             
             def __setitem__(self, key, value):
                 if self.SCHEMA.has_column(key):
-                    if hasattr(self.__class__, key):
+                    if self.SCHEMA.is_conflicting_attribute(self, key):
                         self.META.conflicting_columns[key] = value
                     else:
                         setattr(self, key, value)
@@ -406,11 +415,17 @@ class XRecordDatabase:
 
             def __delattr__(self, key):
                 if self.SCHEMA.has_mtm (key):
-                    if key in self.META.mtm:
-                        self.META.mtm.pop(key)
+                    if key not in self.META.mtm:
+                        self.META.mtm[key] = XRecordMTMList(self, key)
+                    self.META.mtm[key].nullify()
                 elif key in self.SCHEMA.fk:
-                    if key in self.META.references:
-                        del self.META.references[key]
+                    if key not in self.META.references:
+                        self.META.references[key] = XRecordFK(self, key)
+                    self.META.references[key].nullify()
+                elif self.SCHEMA.has_child (key):
+                    if key not in self.META.children:
+                        self.META.children[key] = XRecordChildrenList(self, key)
+                    self.META.children[key].nullify()
 
             def delete(self):
                 """
@@ -636,6 +651,9 @@ class XRecordDatabase:
                 pass
 
             def rename_child_reference(self, old_name, new_name):
+                #if metadata was read from cache, the new name may already be in place
+                if new_name in self.children:
+                    return
                 if old_name in self.children:
                     child = self.children.pop(old_name)
                     self.children[new_name] = child
@@ -643,11 +661,14 @@ class XRecordDatabase:
                     raise Exception("No such child reference: " + old_name)
                 
             def rename_mtm(self, old_name, new_name):
+                #if metadata was read from cache, the new name may already be in place
+                if new_name in self.mtm:
+                    return
                 if old_name in self.mtm:
                     mtm = self.mtm.pop (old_name)
                     self.mtm[new_name] = mtm               
                 else:
-                    raise Exception("No such foreign key: " + old_name )
+                    raise Exception("No such many-to-many relationship key: " + old_name )
                 
             def table_info(self):
                 pass
@@ -680,7 +701,7 @@ class XRecordDatabase:
 
             def default(self, column_name):
                 return self._columns[column_name].COLUMN_DEFAULT
-
+            
             def pre_update(self, xrec, where_condition_dict, update_values_dict):
                 pass
             def post_update(self, xrec):
@@ -693,6 +714,13 @@ class XRecordDatabase:
                 pass
             def post_delete(self, xrec, old_record):
                 pass
+
+            def is_conflicting_attribute(self, xrecord, attribute):
+
+                if hasattr(xrecord.__class__, attribute):
+                    return True
+
+                return False
             
             def set_xrecord_from_record(self, xrecord, record, is_sql=False, **defaults):
                 """Set XRecord fields from a Record instance data"""
@@ -704,7 +732,7 @@ class XRecordDatabase:
                         v = record[c]
                     else:
                         v = defaults.get (c)
-                    if hasattr(xrecord.__class__, c):
+                    if self.is_conflicting_attribute(xrecord, c):
                         xrecord.META.conflicting_columns[c] = self.convert_value_from_sql (xrecord, c, v)
                     else:
                         xrecord[c] = self.convert_value_from_sql (xrecord, c, v)
@@ -713,14 +741,17 @@ class XRecordDatabase:
                     if record:
                         for (c,v) in record:
                             if self.has_column(c): continue
-                            if hasattr(xrecord.__class__, c):
+                            if self.is_conflicting_attribute(xrecord, c):
                                 xrecord.META.conflicting_columns[c] = v
                             else:
                                 xrecord["extra_data_" + c] = v
                                 
                     for (c, v) in defaults.items():
                         if self.has_column(c): continue
-                        xrecord["extra_data_" + c] = v
+                        if self.is_conflicting_attribute(xrecord, c):
+                            xrecord.META.conflicting_columns[c] = v
+                        else:
+                            xrecord["extra_data_" + c] = v
                     
                 xrecord.META.changes.clear()
                 xrecord.META.children.clear()
@@ -796,7 +827,8 @@ class XRecordDatabase:
         self._base_schema_class_ = {}
         self._custom_schema_class_ = {}
         self._schema_instance_ = {}
-        
+
+        self._info = self.InformationSchema
         self.Initialize()
         
     def CustomXRecord(self, tablename):
@@ -888,7 +920,7 @@ class XRecordDatabase:
         return _decorator
             
 
-    def Value2SQL(self, column_name):
+    def Value2SQL(self, table_name, column_name):
         if table_name not in self.value_converters:
             self.value_converters[table_name] = {}
         if column_name not in self.value_converters[table_name]:
@@ -906,10 +938,33 @@ class XRecordDatabase:
         Forget all cached meta data. This may be used after database schema changed, and for
         some reason the application cannot be restarted.
 
-        This is NOT recommended.
+        This is NOT recommended for anything other than internal use in special situations.
         """
         self._schema_instance_.clear()
-    
+
+    @property
+    def MetaData(self):
+        return self.InformationSchema
+
+    def ReadMetaDataCache(self, filename):
+        self.ClearMetaData()
+        try:
+            with open(filename, 'r') as f:
+                return self.MetaData.UnserializeFromFile (f)
+        except IOError:
+            pass
+
+    def WriteMetaDataCache(self, filename):
+        try:
+            with open(filename, 'w') as f:
+                self.MetaData.SerializeToFile (f)
+        except IOError:
+            pass
+
+    @property
+    def InformationSchema(self):
+        return None
+
     def SQLLog(self, stream):
         """
         Set the output stream object, to which all SQL queries run by this database instance are logged.
@@ -974,7 +1029,7 @@ class XRecordDatabase:
         pass
     
     def TableInfo(self, table_name):
-        return self._info.GetForTable(table_name)
+        return self.MetaData.GetForTable(table_name)
 
     @property
     def Connection(self):
@@ -1029,6 +1084,11 @@ class XRecordDatabase:
         updates = ",".join ( map( lambda x: """"{0}" = {1}""".format(x[0], x[1]), values.items()) )
         where = " AND ".join ( map( lambda x: """"{0}" = '{1}'""".format(x[0], x[1]), where.items()) )
         return "UPDATE \"{0}\" SET {1} WHERE {2}".format (table, updates, where)
+
+    @property
+    def Tables(self):
+        for table in self.MetaData.Tables:
+            yield table
         
     def DoSQL(self, sql, *args):
         pass
@@ -1141,7 +1201,7 @@ class XRecordDatabase:
         """
         Run the query, and return the result set as an array of :class:`Record` objects.
         """
-        with self.Query(sql) as result:
+        with self.Query(sql, *args) as result:
             return self.FetchResultsObject(result, **kwargs)
 
     def LazyArrayObject(self, sql, *args, **kwargs):
